package com.yingxin.server.client.config;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;

/**
 * description//TODO
 *
 * @author gaozhentao
 * @version 1.0
 * @date 2019/8/20
 */
@EnableWebSecurity  //开启权限验证
@Configuration
public class SecurityConfig extends WebSecurityConfigurerAdapter {

    @Autowired
    private UserDetailsService myUserDetailsService;

    /**
     * 配置这个bean会在做AuthorizationServerConfigurer配置的时候使用
     * @return
     * @throws Exception
     */
    @Bean
    @Override
    public AuthenticationManager authenticationManagerBean() throws Exception {
        return super.authenticationManagerBean();
    }

    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
        auth.userDetailsService(myUserDetailsService).passwordEncoder(new BCryptPasswordEncoder());
    }

    @Override
    protected void configure(HttpSecurity http) throws Exception {

        http
                // 必须配置，不然OAuth2的http配置不生效
                .requestMatchers()
                .antMatchers( "/oauth/**","/auth/authorize","/auth/login")
                .and()
                .authorizeRequests()
                // 自定义页面或处理url时，如果不配置全局允许，浏览器会提示服务器将页面转发多次
                .antMatchers("/auth/login","/auth/authorize","/oauth/**")
                .permitAll()
                .anyRequest()
                .authenticated();;

        // 表单登录
        http.formLogin()
                // 登录页面
                .loginPage("/auth/login")
                .loginProcessingUrl("/auth/authorize");

        http.httpBasic().disable();
    }

}
