package com.yingxin.server.client.controller;

import com.yingxin.server.client.Service.ClientService;
import com.yingxin.server.client.Service.UserService;
import com.yingxin.server.client.entities.Client;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.client.DefaultResponseErrorHandler;
import org.springframework.web.client.RestTemplate;
import java.io.IOException;
import java.security.Principal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
public class CallbackController {
    @Bean
    public RestTemplate restTemplate(RestTemplateBuilder builder) {
        return builder.build();
    }

    @Autowired
    private RestTemplate restTemplate;

    @Autowired
    ClientService clientService;

    @Autowired
    UserService userService;

    private org.slf4j.Logger logger = LoggerFactory.getLogger(getClass());

    //获取code后，自动提交post请求获取token，取出token，直接获取资源
    @RequestMapping("/oauth/callback")
    public Map getToken(@RequestParam(value = "code") String code, Principal principal){
        //放开后，可获取角色
        //String role=userService.findRoleByUsername(principal.getName()).iterator().next().getRole_name();
        //System.out.println(role);

        String clientId="client1";
        logger.info("receive code {}",code);

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);
        MultiValueMap<String,String> params= new LinkedMultiValueMap<>();

        //获取token
        List<Client> clients=clientService.findClientByCLientId(clientId);
        params.add("grant_type","authorization_code");
        params.add("code",code);
        params.add("client_id",clientId);
        params.add("client_secret",clients.get(0).getClientSecret());
        params.add("redirect_uri",clients.get(0).getWebServerRedirectUri());
        HttpEntity<MultiValueMap<String,String>> requestEntity = new HttpEntity<>(params, headers);
        ResponseEntity<Map> response = restTemplate.postForEntity("http://localhost:8080/oauth/token",requestEntity,Map.class);
        Map token = response.getBody();
        logger.info("token => {}",token);

        //从token中获取access_token
        Map map=new HashMap();
        map.put("access_token",token.get("access_token").toString());
        logger.info("access_token => {}",token.get("access_token").toString());

        //处理401错误
        restTemplate.setErrorHandler(new DefaultResponseErrorHandler() {
            @Override
            public void handleError(ClientHttpResponse response) throws IOException {
                if (response.getRawStatusCode() != 401) {
                    super.handleError(response);
                }
            }
        });

        //提交get请求，获取资源
        ResponseEntity<Map> responseEntity=restTemplate.getForEntity("http://localhost:8088/user?access_token={access_token}",Map.class,map);

        return responseEntity.getBody();
    }
}
